@@ -111,6 +111,13 @@ class BoundaryAxisBase:
         """bool: whether the axis is periodic"""
         return self.grid.periodic[self.axis]
 
+    def get_mathematical_representation(self, field_name: str = "C") -> Tuple[str, str]:
+        """return mathematical representation of the boundary condition"""
+        return (
+            self.low.get_mathematical_representation(field_name),
+            self.high.get_mathematical_representation(field_name),
+        )
+
     def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
         """sets the elements of the sparse representation of this condition
 
