@@ -10,11 +10,10 @@ The module currently supports different boundary conditions:
 * :class:`~pde.grids.boundaries.local.DirichletBC`:
   Imposing the value of a field at the boundary
 * :class:`~pde.grids.boundaries.local.NeumannBC`:
-  Imposing the derivative of a field in the outward normal direction at the
-  boundary
+  Imposing the derivative of a field in the outward normal direction at the boundary
 * :class:`~pde.grids.boundaries.local.MixedBC`:
-  Imposing the derivative of a field in the outward normal direction
-  proportional to its value at the boundary  
+  Imposing the derivative of a field in the outward normal direction proportional to its
+  value at the boundary  
 * :class:`~pde.grids.boundaries.local.CurvatureBC`:
   Imposing the second derivative (curvature) of a field at the boundary
   
@@ -26,6 +25,24 @@ corresponding to the ones listed above are
 :class:`~pde.grids.boundaries.local.MixedNormalBC`, and
 :class:`~pde.grids.boundaries.local.CurvatureNormalBC`.
 
+Finally, there are more specialized classes, which offer greater flexibility, but might
+also require a slightly deeper understanding for proper use:
+
+* :class:`~pde.grids.boundaries.local.ExpressionValueBC`:
+  Imposing the value of a field at the boundary based on a mathematical expression
+* :class:`~pde.grids.boundaries.local.NeumannBC`:
+  Imposing the derivative of a field in the outward normal direction at the boundary
+  based on a mathematical expression
+* :class:`~pde.grids.boundaries.local.UserBC`:
+  Allows full control for setting virtual points, values, or derivatives. The boundary
+  conditions are never enforced automatically. It is thus the user's responsibility to
+  ensure virtual points are set correctly before operators are applied. To set boundary
+  conditions a dictionary :code:`{TARGET: value}` must be supplied as argument `args` to
+  :meth:`set_ghost_cells` or the numba equivalent. Here, `TARGET` determines how the
+  `value` is interpreted and what boundary condition is actually enforced: the value of
+  the virtual points directly (`virtual_point`), the value of the field at the boundary
+  (`value`) or the outward derivative of the field at the boundary (`derivative`).
+
 Note that derivatives are generally given in the direction of the outward normal vector,
 such that positive derivatives correspond to a function that increases across the
 boundary.
@@ -35,6 +52,7 @@ from __future__ import annotations
 
 import logging
 from abc import ABCMeta, abstractmethod
+from numbers import Number
 from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 import numba as nb
@@ -274,6 +292,10 @@ class BCBase(metaclass=ABCMeta):
         else:
             return self.grid.axes_bounds[self.axis][0]
 
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        raise NotImplementedError
+
     @classmethod
     def get_help(cls) -> str:
         """Return information on how boundary conditions can be set"""
@@ -292,9 +314,8 @@ class BCBase(metaclass=ABCMeta):
             "conditions."
         )
 
-    @abstractmethod
     def _repr_value(self) -> List[str]:
-        pass
+        return []
 
     def __repr__(self):
         args = [f"axis={self.axis}", f"upper={self.upper}"]
@@ -393,22 +414,20 @@ class BCBase(metaclass=ABCMeta):
         if "type" in data.keys():
             # type is given (optionally with a value)
             b_type = data.pop("type")
-            b_value = data.pop("value", 0)
+            return cls.from_str(grid, axis, upper, condition=b_type, rank=rank, **data)
 
         elif len(data) == 1:
             # only a single items is given
             b_type, b_value = data.popitem()
+            return cls.from_str(
+                grid, axis, upper, condition=b_type, rank=rank, value=b_value, **data
+            )
 
         else:
             raise BCDataError(
                 f"Boundary conditions `{str(list(data.keys()))}` are not supported."
             )
 
-        # initialize the boundary class with all remaining values forwarded
-        return cls.from_str(
-            grid, axis, upper, condition=b_type, rank=rank, value=b_value, **data
-        )
-
     @classmethod
     def from_data(
         cls, grid: GridBase, axis: int, upper: bool, data: BoundaryData, rank: int = 0
@@ -477,9 +496,14 @@ class BCBase(metaclass=ABCMeta):
     def copy(self, upper: Optional[bool] = None, rank: int = None) -> BCBase:
         pass
 
-    @abstractmethod
     def extract_component(self, *indices):
-        pass
+        """extracts the boundary conditions for the given component
+
+        Args:
+            *indices:
+                One or two indices for vector or tensor fields, respectively
+        """
+        raise NotImplementedError
 
     def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
         raise NotImplementedError
@@ -603,6 +627,171 @@ class BCBase(metaclass=ABCMeta):
         return ghost_cell_setter  # type: ignore
 
 
+class UserBC(BCBase):
+    """represents a boundary whose virtual point are set by the user.
+
+    Boundary conditions will only be set when a dictionary :code:`{TARGET: value}` is
+    supplied as argument `args` to :meth:`set_ghost_cells` or the numba equivalent.
+    Here, `TARGET` determines how the `value` is interpreted and what boundary condition
+    is actually enforced: the value of the virtual points directly (`virtual_point`),
+    the value of the field at the boundary (`value`) or the outward derivative of the
+    field at the boundary (`derivative`).
+
+    Warning:
+        This implies that the boundary conditions are never enforced automatically,
+        e.g., when evaluating an operator. It is thus the user's responsibility to
+        ensure virtual points are set correctly before operators are applied.
+    """
+
+    names = ["user"]
+
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        axis_name = self.grid.axes[self.axis]
+        return f"user-controlled  @ {axis_name}={self.axis_coord}"
+
+    def _cache_hash(self) -> int:
+        """returns a value to determine when a cache needs to be updated"""
+        return hash((self.__class__.__name__, self.grid._cache_hash(), self.axis))
+
+    def copy(
+        self,
+        upper: Optional[bool] = None,
+        rank: int = None,
+    ) -> UserBC:
+        """return a copy of itself, but with a reference to the same grid"""
+        return self.__class__(
+            grid=self.grid,
+            axis=self.axis,
+            upper=self.upper if upper is None else upper,
+            rank=self.rank if rank is None else rank,
+        )
+
+    def set_ghost_cells(self, data_full: np.ndarray, *, args=None) -> None:
+        """set the ghost cell values for this boundary
+
+        Args:
+            data_full (:class:`~numpy.ndarray`):
+                The full field data including ghost points
+            values  (:class:`~numpy.ndarray`):
+                The values determining the values of the ghost cell. The interpretation
+                of this values is determined by `self.target`.
+        """
+        if args is None:
+            # usual case where set_ghost_cells is called automatically. In our case,
+            # won't do anything since we expect the user to call the function manually
+            # with the user data provided as the argument.
+            return
+
+        if any(t in args for t in ["virtual_point", "value", "derivative"]):
+            # ghost cells will only be set if any of the above keys were supplied
+
+            # prepare the array of slices to index bcs
+            offset = data_full.ndim - self.grid.num_axes  # additional data axes
+            idx_offset = [slice(None)] * offset
+            idx_valid = [slice(1, -1)] * self.grid.num_axes
+            idx_write: List[Union[slice, int]] = idx_offset + idx_valid  # type: ignore
+            idx_write[offset + self.axis] = -1 if self.upper else 0
+            idx_read = idx_write[:]
+            idx_read[offset + self.axis] = -2 if self.upper else 1
+
+            # if self.normal:
+            #     assert offset > 0
+            #     idx_write[offset - 1] = self.axis
+            #     idx_read[offset - 1] = self.axis
+
+            # get values right next to the boundary
+            bndry_values = data_full[tuple(idx_read)]
+
+            # calculate the virtual points
+            if "virtual_point" in args:
+                data_full[tuple(idx_write)] = args["virtual_point"]
+            elif "value" in args:
+                data_full[tuple(idx_write)] = 2 * args["value"] - bndry_values
+            elif "derivative" in args:
+                dx = self.grid.discretization[self.axis]
+                data_full[tuple(idx_write)] = dx * args["derivative"] + bndry_values
+            else:
+                raise RuntimeError
+        # else: no-op for the default case where BCs are not set by user
+
+    def make_virtual_point_evaluator(self) -> VirtualPointEvaluator:
+        """returns a function evaluating the value at the virtual support point
+
+        Returns:
+            function: A function that takes the data array and an index marking
+            the current point, which is assumed to be a virtual point. The
+            result is the data value at this point, which is calculated using
+            the boundary condition.
+        """
+        get_arr_1d = _make_get_arr_1d(self.grid.num_axes, self.axis)
+        dx = self.grid.discretization[self.axis]
+
+        @nb.generated_jit(nopython=True)
+        def extract_value(values, arr: np.ndarray, idx: Tuple[int, ...]):
+            """helper function that extracts the correct value from supplied ones"""
+            if isinstance(values, (nb.types.Number, Number)):
+                # scalar was supplied => simply return it
+                def impl(values, arr: np.ndarray, idx: Tuple[int, ...]):
+                    return values
+
+            elif isinstance(arr, (nb.types.Array, np.ndarray)):
+                # array was supplied => extract value at current position
+
+                def impl(values, arr: np.ndarray, idx: Tuple[int, ...]):
+                    _, _, bc_idx = get_arr_1d(arr, idx)
+                    return values[bc_idx]
+
+            else:
+                raise TypeError("Either a scalar or an array must be supplied")
+
+            if nb.config.DISABLE_JIT:
+                print("VALUES", values.__class__)
+                return impl(values, arr, idx)
+            else:
+                return impl
+
+        @register_jitable
+        def virtual_point(arr: np.ndarray, idx: Tuple[int, ...], args):
+            """evaluate the virtual point at `idx`"""
+            if "virtual_point" in args:
+                # set the virtual point directly
+                return extract_value(args["virtual_point"], arr, idx)
+
+            elif "value" in args:
+                # set the value at the boundary
+                value = extract_value(args["value"], arr, idx)
+                return 2 * value - arr[idx]
+
+            elif "derivative" in args:
+                # set the outward derivative at the boundary
+                value = extract_value(args["derivative"], arr, idx)
+                return dx * value + arr[idx]
+
+            else:
+                # no-op for the default case where BCs are not set by user
+                return np.nan
+
+        return virtual_point  # type: ignore
+
+    def make_ghost_cell_setter(self) -> GhostCellSetter:
+        """return function that sets the ghost cells for this boundary"""
+        ghost_cell_setter_inner = super().make_ghost_cell_setter()
+
+        @register_jitable
+        def ghost_cell_setter(data_full: np.ndarray, args=None) -> None:
+            """helper function setting the conditions on all axes"""
+            if args is None:
+                return  # no-op when no specific arguments are given
+
+            if "virtual_point" in args or "value" in args or "derivative" in args:
+                # ghost cells will only be set if any of the above keys were supplied
+                ghost_cell_setter_inner(data_full, args=args)
+            # else: no-op for the default case where BCs are not set by user
+
+        return ghost_cell_setter  # type: ignore
+
+
 class ExpressionBC(BCBase):
     """represents a boundary whose virtual point is calculated from an expression"""
 
@@ -646,9 +835,9 @@ class ExpressionBC(BCBase):
             raise NotImplementedError(
                 "Expression boundary conditions only work for scalar conditions"
             )
-        self._logger = logging.getLogger(self.__class__.__name__)
 
         # determine the full expression for setting the value of the virtual point
+        self._input: Dict[str, Any] = {"expression": value, "target": target}
         if target == "virtual_point":
             expression = value
         elif target == "value":
@@ -677,20 +866,35 @@ class ExpressionBC(BCBase):
             )
 
     def _repr_value(self):
-        return [f'value="{self._expr.expression}"']
+        return [f'{self._input["target"]}="{self._input["expression"]}"']
+
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        axis_name = self.grid.axes[self.axis]
+        target = self._input["target"]
+        expression = self._input["expression"]
+        if target == "virtual_point":
+            return f"{field_name} = {expression}   @ virtual point"
+        elif target == "value":
+            return f"{field_name} = {expression}   @ {axis_name}={self.axis_coord}"
+        elif target == "derivative":
+            if self.upper:
+                return f"∂{field_name}/∂{axis_name} = {expression}   @ {axis_name}={self.axis_coord}"
+            else:
+                return f"-∂{field_name}/∂{axis_name} = {expression}   @ {axis_name}={self.axis_coord}"
+        else:
+            raise NotImplementedError(f"Unsupported target `{target}`")
 
     def _cache_hash(self) -> int:
         """returns a value to determine when a cache needs to be updated"""
-        expression = self._expr.expression
-        return hash(
-            (self.__class__.__name__, self.grid._cache_hash(), self.axis, expression)
-        )
+        data = (self.__class__.__name__, self.grid._cache_hash(), self.axis)
+        return hash(data + tuple(self._input.values()))
 
     def __eq__(self, other):
         """checks for equality neglecting the `upper` property"""
         if not isinstance(other, self.__class__):
             return NotImplemented
-        return super().__eq__(other) and self._expr.expression == other._expr.expression
+        return super().__eq__(other) and self._input == other._input
 
     def copy(
         self,
@@ -703,18 +907,10 @@ class ExpressionBC(BCBase):
             axis=self.axis,
             upper=self.upper if upper is None else upper,
             rank=self.rank if rank is None else rank,
-            value=self._expr.expression,
+            value=self._input["expression"],
+            target=self._input["target"],
         )
 
-    def extract_component(self, *indices):
-        """extracts the boundary conditions for the given component
-
-        Args:
-            *indices:
-                One or two indices for vector or tensor fields, respectively
-        """
-        raise NotImplementedError
-
     def get_data(self, idx: Tuple[int, ...]) -> Tuple[float, Dict[int, float]]:
         raise NotImplementedError
 
@@ -1439,6 +1635,19 @@ class _PeriodicBC(ConstBC1stOrderBase):
         super().__init__(grid, axis, upper)
         self.flip_sign = flip_sign
 
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        if self.upper:
+            other_coord = self.grid.axes_bounds[self.axis][0]
+        else:
+            other_coord = self.grid.axes_bounds[self.axis][1]
+
+        axis_name = self.grid.axes[self.axis]
+        if self.flip_sign:
+            return f"{field_name}({axis_name}={self.axis_coord}) = -{field_name}({axis_name}={other_coord})"
+        else:
+            return f"{field_name}({axis_name}={self.axis_coord}) = {field_name}({axis_name}={other_coord})"
+
     def get_virtual_point_data(self, compiled: bool = False) -> Tuple[Any, Any, int]:
         """return data suitable for calculating virtual points
 
@@ -1478,6 +1687,11 @@ class DirichletBC(ConstBC1stOrderBase):
     names = ["value", "dirichlet"]  # identifiers for this boundary condition
     normal = False
 
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        axis_name = self.grid.axes[self.axis]
+        return f"{field_name} = {self.value}   @ {axis_name}={self.axis_coord}"
+
     def get_virtual_point_data(self, compiled: bool = False) -> Tuple[Any, Any, int]:
         """return data suitable for calculating virtual points
 
@@ -1540,6 +1754,14 @@ class NeumannBC(ConstBC1stOrderBase):
     names = ["derivative", "neumann"]  # identifiers for this boundary condition
     normal = False
 
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        axis_name = self.grid.axes[self.axis]
+        if self.upper:
+            return f"∂{field_name}/∂{axis_name} = {self.value}   @ {axis_name}={self.axis_coord}"
+        else:
+            return f"-∂{field_name}/∂{axis_name} = {self.value}   @ {axis_name}={self.axis_coord}"
+
     def get_virtual_point_data(self, compiled: bool = False) -> Tuple[Any, Any, int]:
         """return data suitable for calculating virtual points
 
@@ -1693,6 +1915,14 @@ class MixedBC(ConstBC1stOrderBase):
             obj.link_value(self.value)
         return obj
 
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        axis_name = self.grid.axes[self.axis]
+        if self.upper:
+            return f"∂{field_name}/∂{axis_name} + {self.value} * {field_name} = {self.const}   @ {axis_name}={self.axis_coord}"
+        else:
+            return f"-∂{field_name}/∂{axis_name} + {self.value} * {field_name} = {self.const}   @ {axis_name}={self.axis_coord}"
+
     def get_virtual_point_data(self, compiled: bool = False) -> Tuple[Any, Any, int]:
         """return data suitable for calculating virtual points
 
@@ -2041,6 +2271,14 @@ class CurvatureBC(ConstBC2ndOrderBase):
     names = ["curvature", "second_derivative", "extrapolate"]  # identifiers for this BC
     normal = False
 
+    def get_mathematical_representation(self, field_name: str = "C") -> str:
+        """return mathematical representation of the boundary condition"""
+        axis_name = self.grid.axes[self.axis]
+        if self.upper:
+            return f"∂²{field_name}/∂{axis_name}² = {self.value}   @ {axis_name}={self.axis_coord}"
+        else:
+            return f"-∂²{field_name}/∂{axis_name}² = {self.value}   @ {axis_name}={self.axis_coord}"
+
     def get_virtual_point_data(
         self,
     ) -> Tuple[np.ndarray, np.ndarray, int, np.ndarray, int]:
