@@ -1,6 +1,6 @@
 """
 Base classes
-   
+
 .. codeauthor:: David Zwicker <david.zwicker@ds.mpg.de> 
 """
 
@@ -487,10 +487,12 @@ class PDEBase(metaclass=ABCMeta):
             dt (float):
                 Time step of the chosen stepping scheme. If `None`, a default value
                 based on the stepper will be chosen. In particular, if
-                `method == 'auto'`, the :class:`~pde.solvers.ScipySolver` with an
-                automatic, adaptive time step is used. This is a flexible choice, but
-                might be slower than using a fixed time step, where the explicit solver
-                :class:`~pde.solvers.explicit.ExplicitSolver` is when `method == 'auto'`.
+                `method == 'auto'`, :class:`~pde.solvers.ScipySolver` with an automatic,
+                adaptive time step provided by scipy is used. This is a flexible choice,
+                but can also result in unstable or slow simulations. If an adaptive
+                stepper is used (supported by :class:`~pde.solvers.ScipySolver` and
+                :class:`~pde.solvers.ExplicitSolver`), the value given here sets the
+                initial time step.
             tracker:
                 Defines a tracker that process the state of the simulation at specified
                 time intervals. A tracker is either an instance of
@@ -504,21 +506,24 @@ class PDEBase(metaclass=ABCMeta):
                 detail. In particular, the interval at which the tracker is evaluated
                 can be chosen when creating a tracker object explicitly.
             method (:class:`~pde.solvers.base.SolverBase` or str):
-                Specifies a method for solving the differential equation. This can
+                Specifies the method for solving the differential equation. This can
                 either be an instance of :class:`~pde.solvers.base.SolverBase` or a
                 descriptive name like 'explicit' or 'scipy'. The valid names are given
                 by :meth:`pde.solvers.base.SolverBase.registered_solvers`. The default
                 value 'auto' selects :class:`~pde.solvers.ScipySolver` if `dt` is not
-                specified and otherwise :class:`~pde.solvers.explicit.ExplicitSolver`.
+                specified and :class:`~pde.solvers.explicit.ExplicitSolver` otherwise.
+                Details of the solvers and additional features (like adaptive time
+                steps) are explained in their documentation.
             ret_info (bool):
                 Flag determining whether diagnostic information about the solver
                 process should be returned.
             **kwargs:
                 Additional keyword arguments are forwarded to the solver class chosen
-                with the `method` argument. In particular, several `schemes` can be
-                selected for the :class:`~pde.solvers.explicit.ExplicitSolver` and the
-                additional arguments of :func:`scipy.integrate.solve_ivp` are accepted
-                for :class:`~pde.solvers.ScipySolver`.
+                with the `method` argument. In particular,
+                :class:`~pde.solvers.explicit.ExplicitSolver` supports several `schemes`
+                and an adaptive stepper can be enabled using :code:`adaptive=True`.
+                Conversely, :class:`~pde.solvers.ScipySolver` accepts the additional
+                arguments of :func:`scipy.integrate.solve_ivp`.
 
         Returns:
             :class:`~pde.fields.base.FieldBase`:
